#include <iostream>
#include <vector>
#include <algorithm>
#include <tuple>

using namespace std;

typedef long long ll;

struct StopPointInfo {
    ll count;
    ll total_cost;
};

struct CostInfo {
    ll cost;
    int from_child_idx;
};

struct Node {
    int id, parent_id;

    ll cost;
    int left_bound, right_bound;

    vector<int> children;
    vector<StopPointInfo> stop_points;
    vector<CostInfo> costs;

    vector<int> children_min_packs;
};

/**
 * returns:
 * 0 if subtree was solved
 * 1 if subtree can't be solved
 */
int solve_recursive(vector<Node> &tree, int current_idx) {
    Node &node = tree[current_idx];

    node.children_min_packs.resize(node.children.size() + 1);

    for (auto child: node.children) {
        if (solve_recursive(tree, child) == 1) {
            return 1;
        }
    }

    if (node.children.empty()) {
        if (node.left_bound != node.right_bound) {
            node.stop_points = {
                    {node.left_bound,  node.left_bound * node.cost},
                    {node.right_bound, node.right_bound * node.cost}
            };
            node.costs = {{node.cost, -1}};
        } else {
            node.stop_points = {{node.left_bound,  node.left_bound * node.cost}};
            node.costs = {};
        }
        node.children_min_packs[0] = node.left_bound;
    } else {

        for (int i = 0; i < node.children.size(); ++i) {
            node.children_min_packs[i + 1] = tree[node.children[i]].stop_points[0].count;
        }

        ll total_size = 1;
        ll current_count = 0;
        ll current_cost = 0;

        for (auto child: node.children) {
            total_size += tree[child].costs.size();
            current_count += tree[child].stop_points[0].count;
            current_cost += tree[child].stop_points[0].total_cost;
        }

        if (current_count > node.right_bound) return 1;

        vector<tuple<ll, int, int>> merged_stop_points(total_size);
        merged_stop_points[0] = {node.cost, -1, -1};
        int cur_idx = 1;

        for (int child_idx = 0; child_idx < node.children.size(); ++child_idx) {
            int child = node.children[child_idx];
            for (int i = 0; i < tree[child].costs.size(); ++i) {
                merged_stop_points[cur_idx++] = {tree[child].costs[i].cost, child_idx, i};
            }
        }

        sort(merged_stop_points.begin(), merged_stop_points.end());

        for (auto &info: merged_stop_points) {
            if (current_count > node.right_bound) return 0;

            ll new_cost = get<0>(info);
            int child_idx = get<1>(info);
            int in_child_idx = get<2>(info);

            ll max_delta_packs;

            if (child_idx == -1) max_delta_packs = 10000000000;
            else {
                Node &child = tree[node.children[child_idx]];
                max_delta_packs = child.stop_points[in_child_idx + 1].count - child.stop_points[in_child_idx].count;
            }

            if (node.stop_points.empty() && current_count >= node.left_bound) {
                node.stop_points.push_back({current_count, current_cost});

            } else if (current_count < node.left_bound && current_count + max_delta_packs >= node.left_bound) {
                ll min_packs = node.left_bound - current_count;

                current_cost += (ll) min_packs * new_cost;
                current_count += min_packs;

                node.stop_points.push_back({node.left_bound, current_cost});

                max_delta_packs -= min_packs;

                node.children_min_packs[child_idx + 1] += min_packs;

            } else if (current_count < node.left_bound) {
                node.children_min_packs[child_idx + 1] += max_delta_packs;
            }

            if (max_delta_packs > 0 && current_count < node.right_bound) {
                ll max_delta = min(max_delta_packs, node.right_bound - current_count);
                current_count += max_delta;
                current_cost += (ll) max_delta * new_cost;

                if (current_count >= node.left_bound) {
                    node.costs.push_back({new_cost, child_idx});
                    node.stop_points.push_back({current_count, current_cost});
                }
            }
        }

        if (current_count < node.left_bound) return 1;
    }

    return 0;
}

void fill_counts_recursive(vector<Node> &tree, vector<int> &counts, int current_idx, int subtree_packs) {
    Node &node = tree[current_idx];

    vector<int> &packs_in_subtrees = node.children_min_packs;

    for (int i = 0; i < node.costs.size(); ++i) {

        packs_in_subtrees[node.costs[i].from_child_idx + 1] += min(
                node.stop_points[i + 1].count - node.stop_points[i].count,
                subtree_packs - node.stop_points[i].count
        );

        if (subtree_packs <= node.stop_points[i + 1].count) break;
    }

    counts[current_idx] = packs_in_subtrees[0];

    for (int i = 0; i < node.children.size(); ++i) {
        fill_counts_recursive(tree, counts, node.children[i], node.children_min_packs[i + 1]);
    }
}

void solve_once() {
    int n;
    cin >> n;

    vector<Node> tree(n);

    tree[0].id = 0;
    tree[0].parent_id = -1;

    for (int i = 1; i < n; ++i) {
        int parent_id;
        cin >> parent_id;
        parent_id -= 1;

        tree[i].id = i;
        tree[i].parent_id = parent_id;

        tree[parent_id].children.push_back(i);
    }

    for (auto &node: tree) {
        cin >> node.cost;
    }
    for (auto &node: tree) {
        cin >> node.left_bound >> node.right_bound;
    }

    if (solve_recursive(tree, 0) == 1) cout << -1 << endl;
    else {
        ll min_cost = tree[0].stop_points[0].total_cost;

        cout << min_cost << "\n";

        vector<int> counts(n);

        fill_counts_recursive(tree, counts, 0, tree[0].stop_points[0].count);

        for (auto c : counts) cout << c << " ";
        cout << endl;
    }
}

int main() {
    int t;
    cin >> t;

    for (int i = 0; i < t; ++i) solve_once();

    return 0;
}
